function est_eq_delta = FHI_CBGPS_delta_est_eq_v2(delta, theta_hat, Y, Z, T, X)
% PURPOSE: evaluate estimating equations of delta (parametric CBGPS).
%--------------------------------------------------------------------------
% USAGE: est_eq_delta = FHI_CBGPS_delta_est_eq_v2(delta, theta_hat, Y, Z, T, X)
% where: delta = parameters (nvar x 1)
%             where nvar = # of regressors
%        theta_hat = estimates of theta ((K+1) x 1) -- theta_hat must be a column vector.
%             where theta_hat(1:K) = beta_hat
%                   theta_hat(K+1) = sigsq_hat 
%                   K = # of covariates
%        Y = outcomes (N x 1)
%             where N = sample size 
%        Z = regressors (N x nvar)
%        T = continuous treatment (N x 1)
%        X = covariates (N x K)
%--------------------------------------------------------------------------
% RETURNS: est_eq_delta = estimating equations (nvar x 1)
%                optimal delta satisfies est_eq_delta = 0.
%--------------------------------------------------------------------------
% References: 
%    Chunrong Ai, Oliver Linton, Kaiji Motegi, and Zheng Zhang (2021). 
%       A unified framework for efficient estimation of general treatment models. 
%       Quantitative Economics, forthcoming. 
%    C. Fong, C. Hazlett, and K. Imai (2018). Covariate balancing
%       propensity score for a continuous treatment: Application to the efficacy
%       of political advertisements. Annals of Applied Statistics, 12, 156-177.
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    February 15, 2021.
% -------------------------------------------------------------------------

N = size(X,1);                    % sample size
K = size(X,2);                    % # of covariates
beta_hat = theta_hat(1:K);        % coefficients of X
sigsq_hat = theta_hat(K+1);       % error variance
nvar = size(Z,2);                 % # of regressors

% standardize T
T_mean = (1/N) * sum(T);                 % 1 x 1 
T_demeaned = T - T_mean;                 % N x 1
s_T = (1/(N-1)) * sum(T_demeaned.^2);    % 1 x 1
T_star = (1/sqrt(s_T)) * T_demeaned;     % N x 1           

% standardize X
X_mean = (1/N) * sum(X);                        % 1 x K
X_demeaned = X - X_mean;                        % N x K
S_X = (1/(N-1)) * (X_demeaned' * X_demeaned);   % K x K
S_X_half = chol(S_X, 'lower');                  % K x K
S_X_half_inv = S_X_half \ eye(K);               % K x K
X_star = X_demeaned * S_X_half_inv';            % N x K

est_eq_delta = zeros(nvar,1);
for i = 1:N
     T_star_i = T_star(i);               % 1 x 1
     X_star_i = X_star(i,:)';            % K x 1
     Y_i = Y(i);                         % 1 x 1
     Z_i = Z(i,:)';                      % nvar x 1
     deviation2 = (T_star_i - X_star_i' * beta_hat)^2;                                     % 1 x 1
     weight_i = sqrt(sigsq_hat) * exp((1/(2*sigsq_hat)) * deviation2 - 0.5 * T_star_i^2);  % 1 x 1
     est_eq_delta_i = weight_i * (Y_i - Z_i' * delta) * Z_i;      % nvar x 1
     est_eq_delta = est_eq_delta + est_eq_delta_i;                % nvar x 1
end    
est_eq_delta = (1/N) * est_eq_delta;     % nvar x 1

end
