function est_eq_theta = FHI_CBGPS_theta_est_eq_v2(theta, T, X)
% PURPOSE: evaluate estimating equations of theta (parametric CBGPS).
%--------------------------------------------------------------------------
% USAGE: est_eq_theta = FHI_CBGPS_theta_est_eq_v2(theta, T, X)
% where: theta = parameters (nparam x 1) -- theta must be a column vector.
%             beta = theta(1:K)      -- coefficients of X
%             sigsq = theta(nparam)  -- error variance
%             nparam = K + 1 
%             K = # of covariates
%        T = continuous treatment (N x 1)
%             N = sample size 
%        X = covariates (N x K)
%--------------------------------------------------------------------------
% RETURNS: est_eq_theta = estimating equations (nparam x 1)
%                optimal theta satisfies est_eq_theta = 0.
%--------------------------------------------------------------------------
% References: 
%    Chunrong Ai, Oliver Linton, Kaiji Motegi, and Zheng Zhang (2021). 
%       A unified framework for efficient estimation of general treatment models. 
%       Quantitative Economics, forthcoming. 
%    C. Fong, C. Hazlett, and K. Imai (2018). Covariate balancing
%       propensity score for a continuous treatment: Application to the efficacy
%       of political advertisements. Annals of Applied Statistics, 12, 156-177.
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    February 15, 2021.
% -------------------------------------------------------------------------

N = size(X,1);            % sample size
K = size(X,2);            % # of covariates
nparam = K + 1;           % # of parameters
beta = theta(1:K);        % coefficients of X
sigsq = theta(nparam);    % error variance

% standardize T
T_mean = (1/N) * sum(T);                % 1 x 1 
T_demeaned = T - T_mean;                % N x 1
s_T = (1/(N-1)) * sum(T_demeaned.^2);   % 1 x 1
T_star = (1/sqrt(s_T)) * T_demeaned;    % N x 1           

% standardize X
X_mean = (1/N) * sum(X);                        % 1 x K
X_demeaned = X - X_mean;                        % N x K
S_X = (1/(N-1)) * (X_demeaned' * X_demeaned);   % K x K
S_X_half = chol(S_X, 'lower');                  % K x K
S_X_half_inv = S_X_half \ eye(K);               % K x K
X_star = X_demeaned * S_X_half_inv';            % N x K

est_eq_theta = zeros(nparam,1);
for i = 1:N
     T_star_i = T_star(i);                                  % 1 x 1
     X_star_i = X_star(i,:)';                               % K x 1
     deviation2 = (T_star_i - X_star_i' * beta)^2;          % 1 x 1
     est_eq_theta_i1 = (1/sigsq) * deviation2 - 1;          % 1 x 1
     est_eq_theta_i2_exp = exp((1/(2*sigsq)) * deviation2 - 0.5 * T_star_i^2);     % 1 x 1
     est_eq_theta_i2 = sqrt(sigsq) * est_eq_theta_i2_exp * T_star_i * X_star_i;    % K x 1
     est_eq_theta_i = [est_eq_theta_i1; est_eq_theta_i2];   % nparam x 1
     est_eq_theta = est_eq_theta + est_eq_theta_i;          % nparam x 1
end    
est_eq_theta = (1/N) * est_eq_theta;   % nparam x 1

end
