function [beta_hat, pihat, mean_pihat] = GOE_bivarX_v1(Y, T, X1, X2, K1, K2, rhotype, options_fminsearch)
% PURPOSE: compute the generalized optimization estimator (GOE) with 
%            two covariates and a linear link function.
%--------------------------------------------------------------------------
% USAGE: [beta_hat, pihat, mean_pihat] = GOE_bivarX_v1(Y, T, X1, X2, K1, K2, rhotype, options_fminsearch)
% where: Y = outcome (N x 1)
%                where N = sample size
%        T = treatment (N x 1)
%        X1 = first covariate (N x 1)
%        X2 = second covariate (N x 1)
%        K1 = dimension of the polynomial with respect to T (1 x 1)
%        K2 = dimension of the polynomial with respect to X (1 x 1)
%        rhotype = 1 for exponential tilting (ET)
%                  2 for empirical likelihood (EL)
%                  3 for quadratic (QR)
%                  4 for inverse logistic (IL)
%        options_fminsearch = options for fminsearch (structure)
%--------------------------------------------------------------------------
% RETURNS: beta_hats = [beta1_hat, beta2_hat]'
%                    = GOE of loadings of linear link function (2 x 1)
%--------------------------------------------------------------------------
% Note: The link function is assumed to be E[Y(t)] = beta0 + beta1 * t.
%       The quadratic loss function is used.
%       beta_hat = argmin sum(pihat * (Y - beta0 - beta1 * T)^2).
%       An analytical solution is available by taking FOCs.
% -------------------------------------------------------------------------
% See also: Lambda_objfun_v5.m, pihat_v4.m, beta_polynomial_closed_form_v1.m
% -------------------------------------------------------------------------
% References: 
%    Chunrong Ai, Oliver Linton, Kaiji Motegi, and Zheng Zhang (2021). 
%       A unified framework for efficient estimation of general treatment models. 
%       Quantitative Economics, forthcoming. 
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    February 15, 2021.
% -------------------------------------------------------------------------

N = length(T);     % sample size

% construct polynomial with respect to T
if K1 == 2
    umat = [ones(N,1), T]';                     % K1 x N
elseif K1 == 3
    umat = [ones(N,1), T, T.^2]';               % K1 x N
elseif K1 == 4
    umat = [ones(N,1), T, T.^2, T.^3]';         % K1 x N
end
          
% construct polynomial with respect to X
if K2 == 3
    vmat = [ones(N,1), X1, X2]';                % K2 x N
elseif K2 == 4
    vmat = [ones(N,1), X1, X2, X1.^2]';         % K2 x N
elseif K2 == 5    
    vmat = [ones(N,1), X1, X2, X1.^2, X2.^2]';        % K2 x N
elseif K2 == 6
    vmat = [ones(N,1), X1, X2, X1.^2, X2.^2, X1.*X2]';        % K2 x N
elseif K2 == 7
    vmat = [ones(N,1), X1, X2, X1.^2, X2.^2, X1.*X2, X1.^3]';        % K2 x N
elseif K2 == 8
    vmat = [ones(N,1), X1, X2, X1.^2, X2.^2, X1.*X2, X1.^3, X2.^3]';        % K2 x N
elseif K2 == 9    
    vmat = [ones(N,1), X1, X2, X1.^2, X2.^2, X1.*X2, X1.^3, X2.^3, (X1.^2).*X2]';        % K2 x N
elseif K2 == 10
    vmat = [ones(N,1), X1, X2, X1.^2, X2.^2, X1.*X2, X1.^3, X2.^3, (X1.^2).*X2, X1.*(X2.^2)]';        % K2 x N
end

% standardize polynomial wrt T
exp_uu = (1/N) * (umat * umat');                % K1 x K1
exp_uu_half = chol(exp_uu, 'lower');
exp_uu_half_inv = exp_uu_half \ eye(K1);
umat_std = exp_uu_half_inv * umat;              % K1 x N

% standardize polynomial wrt X
exp_vv = (1/N) * (vmat * vmat');                % K2 x K2
exp_vv_half = chol(exp_vv, 'lower');
exp_vv_half_inv = exp_vv_half \ eye(K2);
vmat_std = exp_vv_half_inv * vmat;              % K2 x N

Lambda_init = zeros(K1, K2);                    % initial value for Lambda          
Lambda_hat = fminsearch(@Lambda_objfun_v5, Lambda_init, options_fminsearch, umat_std, vmat_std, rhotype);    % find optimal Lambda_hat (K1 x K2)
pihat = pihat_v4(Lambda_hat, umat_std, vmat_std, rhotype);       % compute pi_hat based on Lambda_hat (N x 1)
mean_pihat = (1/N) * sum(pihat);                                 % preparation for distinguishing angel and devil samples (1 x 1)
beta_hat = beta_polynomial_closed_form_v1(Y, T, pihat, 2);       % compute closed-form estimator for beta, assuming the linear link function (2 x 1)

end
