function Neg_Ghat = Lambda_objfun_v5(Lambda, umat_std, vmat_std, rhotype)
% PURPOSE: evaluate the objective function Ghat in order to find an optimal Lambda.
%--------------------------------------------------------------------------
% USAGE: Neg_Ghat = Lambda_objfun_v5(Lambda, umat_std, vmat_std, rhotype)
% where: Lambda = a matrix of parameters of interest (K1 x K2)
%        umat_std = first polynomials (K1 x N)
%                      It must be standardized so that the second moment is eye(K1). 
%        vmat_std = second polynomials (K2 x N)
%                      It must be standardized so that the second moment is eye(K2). 
%                      where N = sample size (scalar)
%        rhotype = 1 for exponential tilting (ET)
%                  2 for empirical likelihood (EL)
%                  3 for quadratic (QR)
%                  4 for inverse logistic (IL)
%--------------------------------------------------------------------------
% RETURNS: Neg_Ghat = objective function (scalar)
%                      optimal Lambda minimizes Neg_Ghat.                    
%--------------------------------------------------------------------------
% SEE ALSO: rho_fun_v2.m.
% -------------------------------------------------------------------------
% References: 
%    Chunrong Ai, Oliver Linton, Kaiji Motegi, and Zheng Zhang (2021). 
%       A unified framework for efficient estimation of general treatment models. 
%       Quantitative Economics, forthcoming. 
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    February 15, 2021.
% -------------------------------------------------------------------------

% default setting
if nargin == 3
    rhotype = 1;            % exponential tilting (ET) 
end    
N = size(umat_std,2);       % sample size

sum_rho = 0;
for i = 1:N
     uvec_std = umat_std(:,i);         % K1 x 1
     vvec_std = vmat_std(:,i);         % K2 x 1
     quad_form_u_Lambda_v = uvec_std' * Lambda * vvec_std;      % this matrix approach is faster than the double-sum approach
     rho_value = rho_fun_v2(quad_form_u_Lambda_v, rhotype);     % we should not code:
                                                                %    [rho_value, ~] = rho_fun_v2(quad_form_u_Lambda_v, rhotype);
                                                                % since this would compute the unnecessary second output too. 
     sum_rho = sum_rho + rho_value;
end
term1 = (1/N) * sum_rho;
ave_u = (1/N) * sum(umat_std,2);       % K1 x 1
ave_v = (1/N) * sum(vmat_std,2);       % K2 x 1
term2 = ave_u' * Lambda * ave_v;       % 1 x 1
Ghat = term1 - term2;
Neg_Ghat = -Ghat;                      % optimal Lambda is supposed to minimize Neg_Ghat.

end
