%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Chunrong Ai, Oliver Linton, Kaiji Motegi, and Zheng Zhang (2021). 
%    A unified framework for efficient estimation of general treatment models. 
%    Quantitative Economics, forthcoming. 
%
% Main Matlab codes for Monte Carlo simulations on interval estimation with a single covariate.
%
% *****************************************************
% Subroutines directly used in this main code:
%    (1) GOE_univX_v1.m
%    (2) Lambda_objfun_v5.m
%    (3) pihat_v4.m
%    (4) beta_polynomial_closed_form_v1.m
%    (5) rho_fun_v2.m
% *****************************************************
%
% Reference: 
%    C. Fong, C. Hazlett, and K. Imai (2018). Covariate balancing
%       propensity score for a continuous treatment: Application to the efficacy
%       of political advertisements. Annals of Applied Statistics, 12, 156-177.
%
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%
% Last updated:
%    February 15, 2021.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
close all
clc

tic;                                           % start timing
start_date = now;                              % start date
disp(['Job started: ', datestr(start_date)]);  % display start date

%% Step 1: Set-up
J = 1000;               % # of Monte Carlo samples
B = 500;                % # of bootstrap samples    
DGP_type = 1;           % 1. DGP-L1: T = 1 + rho_TX * X1 + xi
                        %            Y = 1 + X1 + T + epsilon
                        % 2. DGP-NL1: T = rho_TX * X1^2 + xi
                        %             Y = X1^2 + T + epsilon
N = 100;                % sample size
K1 = 2;                 % K1 = 2, 3, 4
K2 = 3;                 % K2 = 2, 3, 4
                        
if DGP_type == 1                         
    rho_TX = 0.2;
elseif DGP_type == 2
    rho_TX = 0.1; 
end    
                       
rhotype = 1;            % 1 for exponential tilting (ET)
                        % 2 for empirical likelihood (EL)
                        % 3 for quadratic (QR)
                        % 4 for inverse logistic (IL) 
nX = 1;                 % dimension of covariates
sigsq_xi = 1;           % variance of xi
sigsq_eps = 1;          % variance of epsilon
alpha = 0.05;           % nominal size 
options_fminsearch = optimset('Display', 'off');                % options for fminsearch
options_fsolve = optimoptions('fsolve', 'Display', 'off');      % options for fsolve

%% Step 2: Preliminary computation
% compute the link function and true values of parameters of interest
EX = 0;                 % mean of X
EX2 = 1;                % mean of X^2
if DGP_type == 1  
    beta1_true = 1 + EX;
    beta2_true = 1;
elseif DGP_type == 2
    beta1_true = EX2;
    beta2_true = 1;
end    
true_betas = [beta1_true, beta2_true]';         % 2 x 1
p = length(true_betas);

%% Step 3: Run Monte Carlo simulations
CIs_lengths = zeros(2,J,2);
coverages = zeros(2,J,2);
parfor j = 1:J
%for j = 1:J
     xi = sqrt(sigsq_xi) * randn(N,1);          % generate xi (N x 1)
     epsilon = sqrt(sigsq_eps) * randn(N,1);    % generate epsilon (N x 1)
     X = randn(N,1);                            % generate scalar covariate (N x 1)
     
     % generate T and Y
     if DGP_type == 1
         T = 1 + rho_TX * X + xi;               % N x 1
         Y = 1 + X + T + epsilon;               % N x 1
     elseif DGP_type == 2
         T = rho_TX * X.^2 + xi;                % N x 1        
         Y = X.^2 + T + epsilon;                % N x 1
     end     
     
     beta_hat = GOE_univX_v1(Y, T, X, K1, K2, rhotype, options_fminsearch);     % compute GOE (2 x 1)
     
     % generate bootstrap samples
     beta_hat_boot = zeros(2, B);
     for l = 1:B
          index = unidrnd(N, N, 1);             % N x 1
          Y_resample = Y(index);                % N x 1
          T_resample = T(index);                % N x 1     
          X_resample = X(index);                % N x 1
          beta_hat_boot(:,l) = GOE_univX_v1(Y_resample, T_resample, X_resample, K1, K2, rhotype, options_fminsearch);    % compute GOE (2 x 1)
     end 
        
     for i = 1:2
          if i == 1              % asymptotic normality is exploited
              beta_hat_boot_mean = (1/B) * sum(beta_hat_boot, 2);                     % 2 x 1
              se2_hats = (1/B) * sum((beta_hat_boot - beta_hat_boot_mean).^2, 2);     % 2 x 1
              se_hats = sqrt(se2_hats);                             % standard error (2 x 1) 
              CIs_lower = beta_hat - 1.96 * se_hats;                % lower bound of 95% confidence interval (2 x 1)
              CIs_upper = beta_hat + 1.96 * se_hats;                % upper bound of 95% confidence interval (2 x 1)
          elseif i == 2          % asymptotic normality is not exploited
              index_lower = floor(0.025 * B);                       % 1 x 1
              index_upper = floor(0.975 * B);                       % 1 x 1
              beta_hat_boot_sorted = sort(beta_hat_boot, 2);        % 2 x B
              CIs_lower = beta_hat_boot_sorted(:,index_lower);      % lower bound of 95% confidence interval (2 x 1)
              CIs_upper = beta_hat_boot_sorted(:,index_upper);      % upper bound of 95% confidence interval (2 x 1)
          end 
          CIs_length = CIs_upper - CIs_lower;                       % length of 95% confidence interval (w_grid_num x 1) 
          CIs_lengths(:,j,i) = CIs_length;
          condition_lower = (CIs_lower <= true_betas);              % 2 x 1
          condition_upper = (true_betas <= CIs_upper);              % 2 x 1  
          coverage = condition_lower .* condition_upper;            % check coverage (2 x 1)   
          coverages(:,j,i) = coverage;
     end     
end          
   
CP95 = zeros(2,2);               % coverage probability (2 x 2)
ave_length_CI = zeros(2,2);      % mean length (2 x 2)
for i = 1:2
     CP95(:,i) = (1/J) * sum(coverages(:,:,i), 2);                  % 2 x 1
     ave_length_CI(:,i) = (1/J) * sum(CIs_lengths(:,:,i), 2);       % 2 x 1    
end

TABLE_asynorm = [CP95(:,1), ave_length_CI(:,1)];
TABLE_not_asynorm = [CP95(:,2), ave_length_CI(:,2)];

%% Step 4: Report computational time 
time = toc;        % finish timing
end_date = now;    % end date
disp('*****************************************');
disp(['Job started: ', datestr(start_date)]);
disp(['Job finished: ', datestr(end_date)]);
disp(['Computational time: ', num2str(time), ' seconds.']);
disp(['Computational time: ', num2str(time / 60), ' minutes.']);
disp(['Computational time: ', num2str(time / 3600), ' hours.']);
disp('*****************************************');
disp(' ');
