function [bias, stdev, RMSE] = MC_sim_bias_stdev_RMSE_v1(beta_hats, beta0)
% PURPOSE: compute the bias, standard deviation, and RMSE of an estimator 
%           (designed for Monte Carlo simulations)
%--------------------------------------------------------------------------
% USAGE: [bias, stdev, RMSE] = MC_sim_bias_stdev_RMSE_v1(beta_hats, beta0)
% where: beta_hats = estimates across Monte Carlo samples (J x 1)
%              where J = # Monte Carlo iterations
%        beta0 = true value of beta (1 x 1)
%--------------------------------------------------------------------------
% RETURNS: bias = bias (1 x 1)
%          stdev = standard deviation (1 x 1)
%          RMSE = root mean squared error (1 x 1)
%--------------------------------------------------------------------------
% References: 
%    Chunrong Ai, Oliver Linton, Kaiji Motegi, and Zheng Zhang (2021). 
%       A unified framework for efficient estimation of general treatment models. 
%       Quantitative Economics, forthcoming. 
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    February 15, 2021.
% -------------------------------------------------------------------------

J = length(beta_hats);                                     % # of Monte Carlo samples
beta_hat_mean = (1/J) * sum(beta_hats);                    % mean of beta_hat (1 x 1)
bias = beta_hat_mean - beta0;                              % bias (1 x 1)
variance = (1/J) * sum((beta_hats - beta_hat_mean).^2);    % variance (1 x 1)
stdev = sqrt(variance);                                    % standard deviation (1 x 1)
MSE = (1/J) * sum((beta_hats - beta0).^2);                 % mean squared error (1 x 1)
RMSE = sqrt(MSE);                                          % root mean squared error (1 x 1)

end
