%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Chunrong Ai, Oliver Linton, Kaiji Motegi, and Zheng Zhang (2021). 
%    A unified framework for efficient estimation of general treatment models. 
%    Quantitative Economics, forthcoming. 
%
% Main Matlab codes for Monte Carlo simulations on point estimation with a single covariate.
%
% *****************************************************
% Subroutines directly used in this main code:
%    (1) GOE_univX_v1.m
%    (2) Lambda_objfun_v5.m
%    (3) pihat_v4.m
%    (4) beta_polynomial_closed_form_v1.m
%    (5) FHI_CBGPS_theta_est_eq_v2.m
%    (6) FHI_CBGPS_delta_est_eq_v2.m
%    (7) MC_sim_bias_stdev_RMSE_v1.m
%    (8) rho_fun_v2.m
% *****************************************************
%
% Reference: 
%    C. Fong, C. Hazlett, and K. Imai (2018). Covariate balancing
%       propensity score for a continuous treatment: Application to the efficacy
%       of political advertisements. Annals of Applied Statistics, 12, 156-177.
%
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%
% Last updated:
%    February 15, 2021.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
close all
clc

% default settings for figures
% Font settings are particularly important. Without them there will be an 
% error when you try to open eps files via GS View.
set(0, 'DefaultAxesFontName', 'Helvetica');
set(0, 'DefaultTextFontName', 'Helvetica');
set(0, 'DefaultAxesFontSize', 12); 
set(0, 'DefaultTextFontSize', 12); 
set(gcf,'DefaultLineLineWidth', 1.8);

%%%%%%%%%%%%%  Information for saving figures   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%               
directory_name = 'F:\continuous_treatment\matlab\';      % where to save figures
%directory_name = 'E:\continuous_treatment\matlab\';      % where to save figures
%directory_name = 'D:\continuous_treatment\matlab\';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic;                                           % start timing
start_date = now;                              % start date
disp(['Job started: ', datestr(start_date)]);  % display start date

%% Step 1: Set-up
J = 100;                % # of Monte Carlo samples
DGP_type = 1;           % 1. DGP-L1: T = 1 + rho_TX * X1 + xi
                        %            Y = 1 + X1 + T + epsilon
                        % 2. DGP-NL1: T = rho_TX * X1^2 + xi
                        %             Y = X1^2 + T + epsilon
N = 100;                % sample size
method = 10;            % 1. GOE with (K1, K2) = (2, 2)
                        % 2. GOE with (K1, K2) = (2, 3)
                        % 3. GOE with (K1, K2) = (2, 4)
                        % 4. GOE with (K1, K2) = (3, 2)
                        % 5. GOE with (K1, K2) = (3, 3)
                        % 6. GOE with (K1, K2) = (3, 4)
                        % 7. GOE with (K1, K2) = (4, 2)
                        % 8. GOE with (K1, K2) = (4, 3)
                        % 9. GOE with (K1, K2) = (4, 4)
                        % 10. GOE assisted by data-driven selection of K (MSE without penalty)
                        % 11. GOE assisted by data-driven selection of K (MSE with additive penalty)
                        % 12. GOE assisted by data-driven selection of K (MSE with multiplicative penalty)
                        % 13. GOE assisted by data-driven selection of K (5-fold cross validation)
                        % 14. GOE assisted by data-driven selection of K (10-fold cross validation)
                        % 15. GOE assisted by data-driven selection of K (N-fold cross validation, i.e., leave-out CV)
                        % 16. CBGPS
if DGP_type == 1                         
    rho_TX = 0.2;
elseif DGP_type == 2
    rho_TX = 0.1; 
end    
version_number = 14;    % version number of figure
rhotype = 1;            % 1 for exponential tilting (ET)
                        % 2 for empirical likelihood (EL)
                        % 3 for quadratic (QR)
                        % 4 for inverse logistic (IL) 
nX = 1;                 % dimension of covariates
sigsq_xi = 1;           % variance of xi
sigsq_eps = 1;          % variance of epsilon
options_fminsearch = optimset('Display', 'off');                % options for fminsearch
options_fsolve = optimoptions('fsolve', 'Display', 'off');      % options for fsolve

%% Step 2: Preliminary computation
% compute the link function and true values of parameters of interest
EX = 0;                 % mean of X
EX2 = 1;                % mean of X^2
if DGP_type == 1        % DGP-L1: T = 1 + rho_TX * X1 + xi, Y = 1 + X1 + T + epsilon
    beta1_true = 1 + EX;
    beta2_true = 1;
elseif DGP_type == 2    % DGP-NL1: T = rho_TX * X1^2 + xi, Y = X1^2 + T + epsilon
    beta1_true = EX2;
    beta2_true = 1;
end    
true_betas = [beta1_true, beta2_true]';     % 2 x 1
p = length(true_betas);

% set-up for CBGPS
dimbeta = nX;           % dimension of beta (CBGPS)
dimtheta = nX + 1;      % dimension of theta (CBGPS)
dimdelta = nX + 2;      % dimension of delta (CBGPS)

%% Step 3: Run Monte Carlo simulations
num_K1K2 = 9;                 % # of pairs of K1 and K2

mean_pihat = zeros(J,1);
beta1_hats = zeros(J,1);
beta2_hats = zeros(J,1);
optimal_pairs = zeros(J,1);
parfor j = 1:J
%for j = 1:J
     xi = sqrt(sigsq_xi) * randn(N,1);              % generate xi (N x 1)
     epsilon = sqrt(sigsq_eps) * randn(N,1);        % generate epsilon (N x 1)
     X = randn(N,1);                                % generate scalar covariate (N x 1)
     
     % generate T and Y
     if DGP_type == 1          % DGP-L1: T = 1 + rho_TX * X1 + xi, Y = 1 + X1 + T + epsilon
         T = 1 + rho_TX * X + xi;         % N x 1
         Y = 1 + X + T + epsilon;         % N x 1
     elseif DGP_type == 2      % DGP-NL1: T = rho_TX * X1^2 + xi, Y = X1^2 + T + epsilon
         T = rho_TX * X.^2 + xi;          % N x 1        
         Y = X.^2 + T + epsilon;          % N x 1
     end     
     
     if (method <= 9)          % GOE with fixed K
         if method == 1
             K1 = 2;
             K2 = 2;
         elseif method == 2
             K1 = 2;
             K2 = 3;                  
         elseif method == 3
             K1 = 2;
             K2 = 4;                  
         elseif method == 4
             K1 = 3;
             K2 = 2;
         elseif method == 5
             K1 = 3;
             K2 = 3;
         elseif method == 6
             K1 = 3;
             K2 = 4;
         elseif method == 7
             K1 = 4;
             K2 = 2;
         elseif method == 8
             K1 = 4;
             K2 = 3;
         elseif method == 9
             K1 = 4;
             K2 = 4;              
         end    
          
         [beta_hat, ~, mean_pihat(j)] = GOE_univX_v1(Y, T, X, K1, K2, rhotype, options_fminsearch);
         beta1_hats(j) = beta_hat(1);                            % save beta1_hat
         beta2_hats(j) = beta_hat(2);                            % save beta2_hat     
     elseif (method == 10) || (method == 11) || (method == 12)   % GOE with data-driven K (MSE criterion)     
         loss_function = inf;
         for m = 1:num_K1K2      % try each (K1,K2)
              if m == 1
                  K1 = 2;
                  K2 = 2;
              elseif m == 2
                  K1 = 2;
                  K2 = 3;                  
              elseif m == 3
                  K1 = 2;
                  K2 = 4;                  
              elseif m == 4
                  K1 = 3;
                  K2 = 2;
              elseif m == 5
                  K1 = 3;
                  K2 = 3;
              elseif m == 6
                  K1 = 3;
                  K2 = 4;
              elseif m == 7
                  K1 = 4;
                  K2 = 2;
              elseif m == 8
                  K1 = 4;
                  K2 = 3;
              elseif m == 9
                  K1 = 4;
                  K2 = 4;              
              end    
               
              [beta_hat, pihat] = GOE_univX_v1(Y, T, X, K1, K2, rhotype, options_fminsearch);
             
              MSE = (1/N) * sum(pihat .* (Y - beta_hat(1) - beta_hat(2)*T).^2);
              if method == 10
                  loss_function_temp = MSE;                                  % no penalty
              elseif method == 11    
                  loss_function_temp = (1 + 2 * (K1 + K2) / N) * MSE;        % additive penalty
              elseif method == 12    
                  loss_function_temp = (1 + 2 * K1 * K2 / N) * MSE;          % multiplicative penalty
              end    

              if loss_function_temp < loss_function
                  mean_pihat(j) = (1/N) * sum(pihat);               
                  beta1_hats(j) = beta_hat(1);
                  beta2_hats(j) = beta_hat(2);
                  optimal_pairs(j) = m;
                  loss_function = loss_function_temp;
              end    
         end                 
     elseif (method == 13) || (method == 14) || (method == 15)     % cross validation
         if method == 13           % 5-fold CV
             num_groups = 5;
         elseif method == 14       % 10-fold CV
             num_groups = 10;
         elseif method == 15       % leave-out CV
             num_groups = N;
         end    
              
         group_size = N / num_groups;     % size of each group 
         N_est = N - group_size;          % sample size reserved for estimation
         N_pred = group_size;             % sample size reserved for prediction
         
         loss_function = inf;
         for m = 1:num_K1K2               % try each (K1,K2)
              if m == 1
                  K1 = 2;
                  K2 = 2;
              elseif m == 2
                  K1 = 2;
                  K2 = 3;                  
              elseif m == 3
                  K1 = 2;
                  K2 = 4;                  
              elseif m == 4
                  K1 = 3;
                  K2 = 2;
              elseif m == 5
                  K1 = 3;
                  K2 = 3;
              elseif m == 6
                  K1 = 3;
                  K2 = 4;
              elseif m == 7
                  K1 = 4;
                  K2 = 2;
              elseif m == 8
                  K1 = 4;
                  K2 = 3;
              elseif m == 9
                  K1 = 4;
                  K2 = 4;              
              end    
               
              % construct polynomial with respect to T
              if K1 == 2
                  umat = [ones(N,1), T]';                 % K1 x N
              elseif K1 == 3
                  umat = [ones(N,1), T, T.^2]';           % K1 x N
              elseif K1 == 4
                  umat = [ones(N,1), T, T.^2, T.^3]';     % K1 x N
              end
              
              % construct polynomial with respect to X
              if K2 == 2
                  vmat = [ones(N,1), X]';                 % K2 x N
              elseif K2 == 3
                  vmat = [ones(N,1), X, X.^2]';           % K2 x N
              elseif K2 == 4
                  vmat = [ones(N,1), X, X.^2, X.^3]';     % K2 x N
              end

              % standardize polynomial wrt T
              exp_uu = (1/N) * (umat * umat');            % K1 x K1
              exp_uu_half = chol(exp_uu, 'lower');
              exp_uu_half_inv = exp_uu_half \ eye(K1);
             
              % standardize polynomial wrt X
              exp_vv = (1/N) * (vmat * vmat');            % K2 x K2
              exp_vv_half = chol(exp_vv, 'lower');
              exp_vv_half_inv = exp_vv_half \ eye(K2);        
               
              MSE_group = zeros(num_groups,1);
              for h = 1:num_groups
                   first = (h - 1) * group_size + 1;
                   last = h * group_size;

                   % observations for prediction
                   Y_pred = Y(first:last);                % N_pred x 1
                   T_pred = T(first:last);                % N_pred x 1
                   X_pred = X(first:last);                % N_pred x 1  

                   % observations for estimation (well defined for h = 1 and h = num_group)
                   Y_est = Y([1:(first-1), (last+1):end]);       % N_est x 1
                   T_est = T([1:(first-1), (last+1):end]);       % N_est x 1
                   X_est = X([1:(first-1), (last+1):end]);       % N_est x 1

                   % perform estimation
                   % construct polynomial with respect to T
                   if K1 == 2
                       umat_est = [ones(N_est,1), T_est]';                        % K1 x N_est
                   elseif K1 == 3
                       umat_est = [ones(N_est,1), T_est, T_est.^2]';              % K1 x N_est
                   elseif K1 == 4
                       umat_est = [ones(N_est,1), T_est, T_est.^2, T_est.^3]';    % K1 x N_est
                   end

                   % construct polynomial with respect to X
                   if K2 == 2
                       vmat_est = [ones(N_est,1), X_est]';                        % K2 x N_est
                   elseif K2 == 3
                       vmat_est = [ones(N_est,1), X_est, X_est.^2]';              % K2 x N_est
                   elseif K2 == 4
                       vmat_est = [ones(N_est,1), X_est, X_est.^2, X_est.^3]';    % K2 x N_est
                   end

                   umat_std_est = exp_uu_half_inv * umat_est;                     % K1 x N_est           
                   vmat_std_est = exp_vv_half_inv * vmat_est;                     % K2 x N_est               

                   Lambda_init = zeros(K1, K2);                                   % initial value for Lambda
                   Lambda_hat_est = fminsearch(@Lambda_objfun_v5, Lambda_init, options_fminsearch, umat_std_est, vmat_std_est, rhotype);    % find optimal Lambda_hat (K1 x K2)
                   pihat_est = pihat_v4(Lambda_hat_est, umat_std_est, vmat_std_est, rhotype);    % compute pi_hat based on Lambda_hat (N_est x 1)
                   beta_hat_est = beta_polynomial_closed_form_v1(Y_est, T_est, pihat_est, 2);    % compute closed-form estimator for beta, assuming the linear link function (2 x 1)
                   
                   % run prediction
                   % construct polynomial with respect to T
                   if K1 == 2
                       umat_pred = [ones(N_pred,1), T_pred]';                          % K1 x N_pred
                   elseif K1 == 3
                       umat_pred = [ones(N_pred,1), T_pred, T_pred.^2]';               % K1 x N_pred
                   elseif K1 == 4
                       umat_pred = [ones(N_pred,1), T_pred, T_pred.^2, T_pred.^3]';    % K1 x N_pred
                   end
                   
                   % construct polynomial with respect to X
                   if K2 == 2
                       vmat_pred = [ones(N_pred,1), X_pred]';                          % K2 x N_pred
                   elseif K2 == 3
                       vmat_pred = [ones(N_pred,1), X_pred, X_pred.^2]';               % K2 x N_pred
                   elseif K2 == 4
                       vmat_pred = [ones(N_pred,1), X_pred, X_pred.^2, X_pred.^3]';    % K2 x N_pred
                   end               

                   umat_std_pred = exp_uu_half_inv * umat_pred;          % K1 x N_pred
                   vmat_std_pred = exp_vv_half_inv * vmat_pred;          % K2 x N_pred       

                   pihat_pred = pihat_v4(Lambda_hat_est, umat_std_pred, vmat_std_pred, rhotype);     % N_pred x 1
                   MSE_group(h) = (1/N_pred) * sum(pihat_pred .* (Y_pred - beta_hat_est(1) - beta_hat_est(2) * T_pred).^2);    % MSE on the prediction of group h
              end    

              loss_function_temp = (1/num_groups) * sum(MSE_group);    % average MSE across all groups                 
              if loss_function_temp < loss_function
                  mopt = m;
                  loss_function = loss_function_temp;
              end 
         end    
                   
         optimal_pairs(j) = mopt;
         if mopt == 1
             K1 = 2;
             K2 = 2;
         elseif mopt == 2
             K1 = 2;
             K2 = 3;                  
         elseif mopt == 3
             K1 = 2;
             K2 = 4;                  
         elseif mopt == 4
             K1 = 3;
             K2 = 2;
         elseif mopt == 5
             K1 = 3;
             K2 = 3;
         elseif mopt == 6
             K1 = 3;
             K2 = 4;
         elseif mopt == 7
             K1 = 4;
             K2 = 2;
         elseif mopt == 8
             K1 = 4;
             K2 = 3;
         elseif mopt == 9
             K1 = 4;
             K2 = 4;              
         end    
               
         [beta_hat, ~, mean_pihat(j)] = GOE_univX_v1(Y, T, X, K1, K2, rhotype, options_fminsearch);
         beta1_hats(j) = beta_hat(1);                       % save beta1_hat
         beta2_hats(j) = beta_hat(2);                       % save beta2_hat              
     elseif method == 16      % CBGPS
         Z = [ones(N,1), T, X];                             % N x dimdelta
         beta_init = 0.3 * randn(dimbeta,1);                % initial value for beta (dimbeta x 1)
         sigsq_init = 0.5 + rand;                           % initial value for sigsq (1 x 1)
         theta_init = [beta_init; sigsq_init];              % initial value for theta (dimtheta x 1)
         theta_hat = fsolve(@FHI_CBGPS_theta_est_eq_v2, theta_init, options_fsolve, T, X);                    % find optimal theta_hat that satisfies nparam estimating equations

         delta_init = 0.3 * randn(dimdelta,1);              % initial value for delta (dimdelta x 1)
         delta_hat = fsolve(@FHI_CBGPS_delta_est_eq_v2, delta_init, options_fsolve, theta_hat, Y, Z, T, X);   % find optimal delta_hat that satisfies nvar estimating equations
         delta_hat_12 = delta_hat(1:2);                     % key parameters that determine link function (2 x 1)
         beta1_hats(j) = delta_hat_12(1);
         beta2_hats(j) = delta_hat_12(2);          
     end
end          
   
[Bias1, Stdev1, RMSE1] = MC_sim_bias_stdev_RMSE_v1(beta1_hats, beta1_true);    % summary statistics of beta1_hat
[Bias2, Stdev2, RMSE2] = MC_sim_bias_stdev_RMSE_v1(beta2_hats, beta2_true);    % summary statistics of beta2_hat
     
BIAS = [Bias1, Bias2];                                            % save bias   
STDEV = [Stdev1, Stdev2];                                         % save standard deviation
RMSE = [RMSE1, RMSE2];                                            % save RMSE
TABLE = [BIAS', STDEV', RMSE'];

devil_index = find((mean_pihat < 0.5) | (mean_pihat > 2) | isnan(mean_pihat));  
angel_index = find((mean_pihat >= 0.5) & (mean_pihat <= 2) & ~isnan(mean_pihat));
J_angel = size(angel_index,1);
beta1_hats_angel = beta1_hats(angel_index);
beta2_hats_angel = beta2_hats(angel_index);

[Bias1_angel, Stdev1_angel, RMSE1_angel] = MC_sim_bias_stdev_RMSE_v1(beta1_hats_angel, beta1_true);    % summary statistics of beta1_hat_angel
[Bias2_angel, Stdev2_angel, RMSE2_angel] = MC_sim_bias_stdev_RMSE_v1(beta2_hats_angel, beta2_true);    % summary statistics of beta2_hat_angel

BIAS_angel = [Bias1_angel, Bias2_angel];                          % save bias   
STDEV_angel = [Stdev1_angel, Stdev2_angel];                       % save standard deviation
RMSE_angel = [RMSE1_angel, RMSE2_angel];                          % save RMSE
TABLE_angel = [BIAS_angel', STDEV_angel', RMSE_angel'];

%% Step 4: Draw histograms
shares = zeros(num_K1K2, 1);
if (method >= 10) && (method <= 15)     % data-driven (K1, K2)
    for m = 1:num_K1K2
         shares(m,:) = (1/J) * sum(optimal_pairs == m);
    end 
    
    figure(1)
    bar(shares, 'FaceColor', [0, 0.447, 0.741]);       % need to fix color since different versions of Matlab may have different default colors
    xticks(1:num_K1K2);
    xticklabels({'(2,2) ', '(2,3) ', '(2,4) ', '(3,2) ', '(3,3) ', '(3,4) ', '(4,2) ', '(4,3) ', '(4,4) '});
    ylim([0,1]);
    grid on
    if DGP_type == 1
        if method == 10
            file_name = strcat('bars_share_N', num2str(N), '_L1_MSE_no_penal_v', num2str(version_number));
        elseif method == 11
            file_name = strcat('bars_share_N', num2str(N), '_L1_MSE_add_penal_v', num2str(version_number));
        elseif method == 12
            file_name = strcat('bars_share_N', num2str(N), '_L1_MSE_multi_penal_v', num2str(version_number));
        elseif method == 13
            file_name = strcat('bars_share_N', num2str(N), '_L1_CV5_v', num2str(version_number));
        elseif method == 14
            file_name = strcat('bars_share_N', num2str(N), '_L1_CV10_v', num2str(version_number));
        elseif method == 15
            file_name = strcat('bars_share_N', num2str(N), '_L1_LOCV_v', num2str(version_number));
        end    
    elseif DGP_type == 2
        if method == 10
            file_name = strcat('bars_share_N', num2str(N), '_NL1_MSE_no_penal_v', num2str(version_number));
        elseif method == 11
            file_name = strcat('bars_share_N', num2str(N), '_NL1_MSE_add_penal_v', num2str(version_number));
        elseif method == 12
            file_name = strcat('bars_share_N', num2str(N), '_NL1_MSE_multi_penal_v', num2str(version_number));
        elseif method == 13
            file_name = strcat('bars_share_N', num2str(N), '_NL1_CV5_v', num2str(version_number));
        elseif method == 14
            file_name = strcat('bars_share_N', num2str(N), '_NL1_CV10_v', num2str(version_number));
        elseif method == 15
            file_name = strcat('bars_share_N', num2str(N), '_NL1_LOCV_v', num2str(version_number));
        end                      
    end
    print('-depsc2', strcat(directory_name, file_name));
end    
    
%% Step 5: Report computational time 
time = toc;        % finish timing
end_date = now;    % end date
disp('*****************************************');
disp(['Job started: ', datestr(start_date)]);
disp(['Job finished: ', datestr(end_date)]);
disp(['Computational time: ', num2str(time), ' seconds.']);
disp(['Computational time: ', num2str(time / 60), ' minutes.']);
disp(['Computational time: ', num2str(time / 3600), ' hours.']);
disp('*****************************************');
disp(' ');
