function beta_hat = beta_polynomial_closed_form_v1(Y, T, pihat, p)
% PURPOSE: estimate parameters in a link function of order p.
%--------------------------------------------------------------------------
% USAGE: beta_hat = beta_polynomial_closed_form_v1(Y, T, pihat, p)
% where: Y = outcome (N x 1)
%        T = treatment (N x 1)
%        pihat = estimated wegiths (N x 1)
%                where N = sample size
%        p = # of parameters (i.e., the dimension of beta)
%--------------------------------------------------------------------------
% RETURNS: beta_hat = [beta1_hat, ..., betap_hat]'
%                   = estimates of parameters of the link function (p x 1)
%--------------------------------------------------------------------------
% Note: The link function is assumed to be 
%          E[Y(t)] = beta1 + beta2 * t + ... + betap * t^{p-1}.
%       The quadratic loss function is used.
%       beta_hat = argmin sum(pihat * (Y - beta1 - beta2 * T - ... - betap * T^{p-1})^2).
%       An analytical solution is available by taking FOCs.
% -------------------------------------------------------------------------
% References: 
%    Chunrong Ai, Oliver Linton, Kaiji Motegi, and Zheng Zhang (2021). 
%       A unified framework for efficient estimation of general treatment models. 
%       Quantitative Economics, forthcoming. 
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    February 15, 2021.
% -------------------------------------------------------------------------

N = size(Y,1);                                           % sample size (1 x 1)
T_poly_mat = repmat(T, 1, p).^repmat(0:(p-1), N, 1);     % N x p
Pimat = diag(pihat);                                     % N x N
piTT = T_poly_mat' * Pimat * T_poly_mat;                 % p x p
piTT_inv = piTT \ eye(p);                                % p x p
piY = repmat(pihat .* Y, 1, p);                          % N x p
piYT = sum(piY .* T_poly_mat)';                          % p x 1
beta_hat = piTT_inv * piYT;                              % p x 1
 
end
