function beta_hat = betahat_binary_v1(Y, D, pihat)
% PURPOSE: estimate parameters in a link function for the binary case.
%--------------------------------------------------------------------------
% USAGE: beta_hat = betahat_binary_v1(Y, D, pihat)
% where: Y = outcome (N x 1)
%        D = binary treatment (N x 1)
%        pihat = estimated wegiths (N x 1)
%                where N = sample size
%--------------------------------------------------------------------------
% RETURNS: beta_hat = [beta1_hat, beta2_hat]'
%                   = estimates of parameters of the link function (2 x 1)
%--------------------------------------------------------------------------
% Note: The link function is assumed to be 
%          E[Y(t)] = beta1 + beta2 * D(t),
%       where D(t) = 1 if t exceeds a certain threshold and D(t) = 0 otherwise.
%       The quadratic loss function is used.
% -------------------------------------------------------------------------
% References: 
%    Chunrong Ai, Oliver Linton, Kaiji Motegi, and Zheng Zhang (2021). 
%       A unified framework for efficient estimation of general treatment models. 
%       Quantitative Economics, forthcoming. 
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    February 15, 2021.
% -------------------------------------------------------------------------

% compute beta1_hat
beta1_hat_num = sum((1 - D) .* pihat .* Y);                       % 1 x 1
beta1_hat_den = sum((1 - D) .* pihat);                            % 1 x 1 
beta1_hat = beta1_hat_num / beta1_hat_den;                        % 1 x 1

% compute beta2_hat
beta2_hat_term1_num = sum(D .* pihat .* Y);                       % 1 x 1
beta2_hat_term1_den = sum(D .* pihat);                            % 1 x 1
beta2_hat_term1 = beta2_hat_term1_num / beta2_hat_term1_den;      % 1 x 1
beta2_hat = beta2_hat_term1 - beta1_hat;                          % 1 x 1
 
beta_hat = [beta1_hat, beta2_hat]';                               % 2 x 1

end
