%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Chunrong Ai, Oliver Linton, Kaiji Motegi, and Zheng Zhang (2021). 
%    A unified framework for efficient estimation of general treatment models. 
%    Quantitative Economics, forthcoming. 
%
% Main Matlab codes for empirical applications (binary treatment).
%
% *****************************************************
% Subroutines used in this main code:
%    (1) lambda1_binary_objfun_v1.m 
%    (2) lambda2_binary_objfun_v1.m 
%    (3) pihat_binary_v1.m
%    (4) betahat_binary_v1.m
%    (5) rho_fun_v2.m
%
% Data:
%    (1) zip_ads_v4.txt
% *****************************************************
%
% Reference: 
%    C. Fong, C. Hazlett, and K. Imai (2018). Covariate balancing
%       propensity score for a continuous treatment: Application to the efficacy
%       of political advertisements. Annals of Applied Statistics, 12, 156-177.
%
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%
% Last updated:
%    February 15, 2021.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
close all;
clc;

tic;                                           % start timing
start_date = now;                              % start date
disp(['Job started: ', datestr(start_date)]);  % display start date

%% Step 1: Set-up
B = 1000;              % # of bootstrap samples  
rhotype = 1;           % 1 for exponential tilting (ET)
                       % 2 for empirical likelihood (EL)
                       % 3 for quadratic (QR)
                       % 4 for inverse logistic (IL) 
threshold = 4;         % cut-off value for dichotomization
alpha = 0.05;          % nominal size

options_fminsearch = optimset('Display','off');              % options for fminsearch
options_fsolve = optimoptions('fsolve','Display','off');     % options for fsolve

% fix seed so that we always get the same result
defaultStream = RandStream.getGlobalStream;
defaultStream.reset(3);

%% Step 2: Empirical analysis (point estimation)
load zip_ads_v4.txt
zip = zip_ads_v4(:,1);                         % zip code
ads = zip_ads_v4(:,2);                         % # of political advertisements aired in each zipcode (treatment of interest)
pop = zip_ads_v4(:,3);                         % population
pct_over65 = zip_ads_v4(:,4);                  % percent over age 65
median_income = zip_ads_v4(:,5);               % median household income
pct_black = zip_ads_v4(:,6);                   % percent black
pct_hispanic = zip_ads_v4(:,7);                % percent hispanic
pop_density = zip_ads_v4(:,8);                 % population density
pct_colgrads = zip_ads_v4(:,9);                % percent college graduates
can_commute = zip_ads_v4(:,10);                % binary indicator of whether it is possible to commute to the zip code from a competitive state
contribution = zip_ads_v4(:,11);               % total amount of campaign contribution (Y)
log_pop = log(pop);                            % log of population
log_median_income = log(median_income + 1);    % some elements of median_income are 0
log_pop_density = log(pop_density + 1);        % log of population density

Y = log(contribution + 1);                     % outcome 
T = log(ads + 1);                              % treatment
D = (T >= threshold);                          % binary treatment
X = [log_pop, log_pop_density, log_median_income, pct_over65, pct_hispanic, pct_black, pct_colgrads, can_commute];     % covariates
dimX = size(X,2);                              % dimension of covariates
N = size(zip,1);                               % sample size
vmat = [ones(1,N); X'];                        % compute polynomial wrt X (K x N)   
K = size(vmat,1);                              % dimension of polynomial

% standardize polynomial wrt X
exp_vv = (1/N) * (vmat * vmat');               % K x K
exp_vv_half = chol(exp_vv, 'lower');
exp_vv_half_inv = exp_vv_half \ eye(K);
vmat_std = exp_vv_half_inv * vmat;             % K x N

lambda_init = zeros(K,1);
lambda1_hat = fminsearch(@lambda1_binary_objfun_v1, lambda_init, options_fminsearch, D, vmat_std, rhotype);    % find optimal lambda1_hat (K x 1)
lambda2_hat = fminsearch(@lambda2_binary_objfun_v1, lambda_init, options_fminsearch, D, vmat_std, rhotype);    % find optimal lambda2_hat (K x 1)
pihat = pihat_binary_v1(lambda1_hat, lambda2_hat, D, vmat_std, rhotype);     % N x 1

% Under the quadratic loss function, an analytical solution is available
beta_hat = betahat_binary_v1(Y, D, pihat);                                   % 2 x 1

%% Step 3: Empirical analysis (interval estimation)
beta_hat_boot = zeros(2, B);
parfor l = 1:B
     index = unidrnd(N, N, 1);                   % N x 1
     Y_resample = Y(index);                      % N x 1
     D_resample = D(index);                      % N x 1     
     X_resample = X(index,:);                    % N x dimX
     vmat = [ones(1,N); X_resample'];            % compute polynomial wrt X (K x N)   
     K = size(vmat,1);   
 
     % standardize polynomial wrt X
     exp_vv = (1/N) * (vmat * vmat');            % K x K
     exp_vv_half = chol(exp_vv, 'lower');
     exp_vv_half_inv = exp_vv_half \ eye(K);
     vmat_std = exp_vv_half_inv * vmat;          % K x N

     lambda_init = zeros(K,1);
     lambda1_hat = fminsearch(@lambda1_binary_objfun_v1, lambda_init, options_fminsearch, D_resample, vmat_std, rhotype);    % find optimal lambda1_hat (K x 1)
     lambda2_hat = fminsearch(@lambda2_binary_objfun_v1, lambda_init, options_fminsearch, D_resample, vmat_std, rhotype);    % find optimal lambda2_hat (K x 1)
     pihat = pihat_binary_v1(lambda1_hat, lambda2_hat, D_resample, vmat_std, rhotype);     % N x 1
     beta_hat_boot(:,l) = betahat_binary_v1(Y_resample, D_resample, pihat);                % 2 x 1
end
     
% construct bootstrapped confidence interval
index_lower = floor(0.5 * alpha * B);                 % 1 x 1
index_upper = floor((1 - 0.5 * alpha) * B);           % 1 x 1
beta_hat_boot_sorted = sort(beta_hat_boot, 2);        % p x B
CIs_lower = beta_hat_boot_sorted(:,index_lower);      % lower bound of 95% confidence interval (p x 1)
CIs_upper = beta_hat_boot_sorted(:,index_upper);      % upper bound of 95% confidence interval (p x 1)
CIs = [CIs_lower, CIs_upper];                         % 95% confidence interval (p x 2)

%% Step 4: Report computational time 
time = toc;        % finish timing
end_date = now;    % end date
disp('*****************************************');
disp(['Job started: ', datestr(start_date)]);
disp(['Job finished: ', datestr(end_date)]);
disp(['Computational time: ', num2str(time), ' seconds.']);
disp(['Computational time: ', num2str(time / 60), ' minutes.']);
disp(['Computational time: ', num2str(time / 3600), ' hours.']);
disp('*****************************************');
disp(' ');
