function Neg_Ghat = lambda2_binary_objfun_v1(lambda2, D, vmat_std, rhotype)
% PURPOSE: evaluate the objective function in order to find an optimal lambda2
%          (binary case).
%--------------------------------------------------------------------------
% USAGE: Neg_Ghat = lambda2_binary_objfun_v1(lambda2, D, vmat_std, rhotype)
% where: lambda2 = a matrix of parameters of interest (K x 1)
%                     where K is the dimension of the sieve basis function       
%        D = binary treatment (N x 1)
%                     where N = sample size
%        vmat_std = sieve basis function (K x N)
%                     It must be standardized so that the second moment is eye(K).
%        rhotype = 1 for exponential tilting (ET)
%                  2 for empirical likelihood (EL)
%                  3 for quadratic (QR)
%                  4 for inverse logistic (IL)
%--------------------------------------------------------------------------
% RETURNS: Neg_Ghat = objective function (1 x 1)
%                      optimal lambda2 is the minimizer of Neg_Ghat.                    
%--------------------------------------------------------------------------
% SEE ALSO: rho_fun_v2.m.
% -------------------------------------------------------------------------
% References: 
%    Chunrong Ai, Oliver Linton, Kaiji Motegi, and Zheng Zhang (2021). 
%       A unified framework for efficient estimation of general treatment models. 
%       Quantitative Economics, forthcoming. 
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    February 15, 2021.
% -------------------------------------------------------------------------

% default setting
if nargin == 3
    rhotype = 1;            % exponential tilting (ET) 
end    

N = size(vmat_std,2);       % sample size
rhos = zeros(N,1);
for i = 1:N
     rhos(i) = rho_fun_v2(lambda2' * vmat_std(:,i), rhotype);
end    
Ghat_term1 = sum((1-D) .* rhos) / sum(1-D);          % 1 x 1
Ghat_term2 = (1/N) * sum(lambda2' * vmat_std);       % 1 x 1
Ghat = Ghat_term1 - Ghat_term2;                      % 1 x 1
Neg_Ghat = -Ghat;                                    % optimal lambda2 is the minimizer of Neg_Ghat (1 x 1).

end
