function pihat = pihat_binary_v1(lambda1, lambda2, D, vmat_std, rhotype)
% PURPOSE: compute a vector of pi_hats in the case of binary treatment.
%--------------------------------------------------------------------------
% USAGE: pihat = pihat_binary_v1(lambda1, lambda2, D, vmat_std, rhotype)
% where: lambda1 = lambdas for treated (K x 1)
%        lambda2 = lambdas for non-treated (K x 1)
%        D = binary treatment (N x 1)
%        vmat_std = sieve basis function (K x N)
%                     It must be standardized so that the second moment is eye(K), 
%                     where N = sample size (1 x 1)
%        rhotype = 1 for exponential tilting (ET)
%                  2 for empirical likelihood (EL)
%                  3 for quadratic (QR)
%                  4 for inverse logistic (IL)
%--------------------------------------------------------------------------
% RETURNS: pihat = vector of pi_hats (N x 1)
%--------------------------------------------------------------------------
% SEE ALSO: rho_fun_v2.m.
% -------------------------------------------------------------------------
% References: 
%    Chunrong Ai, Oliver Linton, Kaiji Motegi, and Zheng Zhang (2021). 
%       A unified framework for efficient estimation of general treatment models. 
%       Quantitative Economics, forthcoming. 
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    February 15, 2021.
% -------------------------------------------------------------------------

N = size(D,1);        % sample size
pihat = zeros(N,1); 
for i = 1:N
     vvec_std = vmat_std(:,i);            % K x 1
     [~, pihat1_temp] = rho_fun_v2(sum(lambda1 .* vvec_std), rhotype);    % derivative of rho
     pihat1 = D(i) * pihat1_temp;
     [~, pihat2_temp] = rho_fun_v2(sum(lambda2 .* vvec_std), rhotype);    % derivative of rho
     pihat2 = (1 - D(i)) * pihat2_temp;
     pihat(i) = pihat1 + pihat2;
end    
 
end
