function pihat = pihat_v4(Lambda, umat_std, vmat_std, rhotype)
% PURPOSE: compute a vector of pi_hats.
%--------------------------------------------------------------------------
% USAGE: pihat = pihat_v4(Lambda, umat_std, vmat_std, rhotype)
% where: Lambda = a matrix of lambdas (K1 x K2)
%        umat_std = polynomials wrt T (K1 x N)
%                     It must be standardized so that the second moment is eye(K1). 
%        vmat_std = second polynomials (K2 x N)
%                     It must be standardized so that the second moment is eye(K2). 
%                     where N = sample size (scalar)
%        rhotype = 1 for exponential tilting (ET)
%                  2 for empirical likelihood (EL)
%                  3 for quadratic (QR)
%                  4 for inverse logistic (IL)
%--------------------------------------------------------------------------
% RETURNS: pihat = vector of pi_hats (N x 1)
%--------------------------------------------------------------------------
% SEE ALSO: rho_fun_v2.m.
% -------------------------------------------------------------------------
% References: 
%    Chunrong Ai, Oliver Linton, Kaiji Motegi, and Zheng Zhang (2021). 
%       A unified framework for efficient estimation of general treatment models. 
%       Quantitative Economics, forthcoming. 
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    February 15, 2021.
% -------------------------------------------------------------------------

N = size(umat_std, 2);      % sample size
pihat = zeros(N,1); 
for i = 1:N
     uvec_std = umat_std(:,i);            % K1 x 1
     vvec_std = vmat_std(:,i);            % K2 x 1
     quad_form_u_Lambda_v = uvec_std' * Lambda * vvec_std;         % this matrix approach is faster than the double-sum approach
     [~, pihat(i)] = rho_fun_v2(quad_form_u_Lambda_v, rhotype);    % derivative of rho
end    
 
end
