function [rho_value, rho_deriv_value] = rho_fun_v2(nu, rhotype)
% PURPOSE: evaluate the rho-function.
%--------------------------------------------------------------------------
% USAGE: [rho_value, rho_deriv_value] = rho_fun_v2(nu, rhotype)
% where: nu = arbitrary argument (scalar) 
%        rhotype = 1 for exponential tilting (ET)
%                  2 for empirical likelihood (EL)
%                  3 for quadratic (QR)
%                  4 for inverse logistic (IL)
%--------------------------------------------------------------------------
% RETURNS: rho_value = level (scalar)
%          rho_deriv_value = first derivative (scalar)
% --------------------------------------------------------------------------
% NOTE: Exponential tilting (ET) is the only option that has a theoretical
%          justification.
% --------------------------------------------------------------------------
% References: 
%    Chunrong Ai, Oliver Linton, Kaiji Motegi, and Zheng Zhang (2021). 
%       A unified framework for efficient estimation of general treatment models. 
%       Quantitative Economics, forthcoming. 
% -------------------------------------------------------------------------
% Written by:
%    Kaiji Motegi
%    Associate professor
%    Graduate School of Economics, Kobe University
%--------------------------------------------------------------------------
% Last updated:
%    February 15, 2021.
% -------------------------------------------------------------------------

if rhotype == 1         % exponential tilting (ET) - with a theoretical justification
    if nargout == 1
       rho_value = -exp(-nu);             % level
    elseif nargout == 2   
       rho_value = -exp(-nu);             % level
       rho_deriv_value = exp(-nu);        % first derivative   
    end
elseif rhotype == 2     % empirical likelihood (EL)
    if nargout == 1
       rho_value = log(1 + nu);           % level
    elseif nargout == 2   
       rho_value = log(1 + nu);           % level
       rho_deriv_value = (1 + nu)^(-1);   % first derivative   
    end    
elseif rhotype == 3     % quadratic (QR)
    if nargout == 1
       rho_value = -0.5 * (1 - nu)^2;     % level
    elseif nargout == 2   
       rho_value = -0.5 * (1 - nu)^2;     % level
       rho_deriv_value = 1 - nu;          % first derivative       
    end          
elseif rhotype == 4     % inverse logistic (IL)
    if nargout == 1
       rho_value = nu - exp(-nu);         % level
    elseif nargout == 2   
       rho_value = nu - exp(-nu);         % level
       rho_deriv_value = 1 + exp(-nu);    % first derivative    
    end          
end    
    
end
